
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerConfigurationException;
import com.icl.saxon.style.*;
import com.icl.saxon.Context;
import com.icl.saxon.expr.Expression;
import com.icl.saxon.output.Outputter;
import com.icl.saxon.om.ExtendedAttributes;

public class SAXONExtensions implements ExtensionElementFactory {

  public Class getExtensionClass(String localname) {
    try {
      if (localname.equals("debug")) 
	return Class.forName("SAXONExtensions$DebugElement");    
      else if (localname.equals("insert-time"))
	return Class.forName("SAXONExtensions$InsertTimeElement");
      else if (localname.equals("time"))
	return Class.forName("SAXONExtensions$TimeElement");
    }
    catch (ClassNotFoundException e) {
    }

    return null;
  }

  
  /**
   * An element that appends the string value of a specified XPath
   * expression to a named file. The expression must be in the
   * attribute 'select' and the file name in the attribute 'file'.
   */
  
  public static class DebugElement extends StyleElement {
    private String filename;
    private Expression select;
    
    public boolean isInstruction() {
      return true;
    }

    public void prepareAttributes()
      throws TransformerConfigurationException {
      ExtendedAttributes atts = getAttributeList();
      filename = atts.getValue("file");
      if (filename == null)
	reportAbsence("file");
      String selectValue = atts.getValue("select");
      if (selectValue == null)
	reportAbsence("select");
      select = makeExpression(selectValue);
    }

    public void validate() throws TransformerConfigurationException {
      checkEmpty();
      checkWithinTemplate();
    }

    public void process(Context context) throws TransformerException {
      try {
	FileWriter out = new FileWriter(filename, true);
	out.write(select.evaluateAsString(context));
	out.close();
      }
      catch (IOException e) {
	throw new TransformerException(e);
      }
    }
  }

  
  /**
   * An element that inserts the current time, formatted using an
   * optional format expression. The expression, if given, is given
   * in the 'format' attribute.
   */
  
  public static class InsertTimeElement extends StyleElement {
    private SimpleDateFormat format;
    
    public boolean isInstruction() {
      return true;
    }

    public void prepareAttributes()
      throws TransformerConfigurationException {
      ExtendedAttributes atts = getAttributeList();
      String formatValue = atts.getValue("format");
      if (formatValue != null)
	format = new SimpleDateFormat(formatValue);
    }

    public void validate() throws TransformerConfigurationException {
      checkEmpty();
      checkWithinTemplate();
    }

    public void process(Context context) throws TransformerException {
      String date;
      if (format != null) 
	date = format.format(new java.util.Date());
      else
	date = new java.util.Date().toString();
      
      Outputter out = context.getOutputter();
      out.writeContent(date);
    }
  }

  
  /**
   * An element that can be used to measure the performance of
   * XSLT stylesheets. It measures the time it takes to execute
   * its contents and outputs this to the standard output. In
   * case there are several time elements in the stylesheet the
   * element can be given a name in the 'name' attribute.
   */
  
  public static class TimeElement extends StyleElement {
    private String name;
    private int times;
    private double timeTotal;
    
    public boolean isInstruction() {
      return true;
    }

    public boolean mayContainTemplateBody() {
      return true;
    }

    public void prepareAttributes()
      throws TransformerConfigurationException {
      ExtendedAttributes atts = getAttributeList();
      name = atts.getValue("name");
    }

    public void validate() throws TransformerConfigurationException {
      checkWithinTemplate();
    }

    public void process(Context context) throws TransformerException {
      double start = System.currentTimeMillis();
      processChildren(context);
      double time = System.currentTimeMillis() - start;

      timeTotal += time;
      times++;
      System.out.println("Executed " + name + " in " + (time / 1000) +
			 "seconds");
      System.out.println("Average: " + (timeTotal / times) +
                         " (" + times + ")");
    }
  }
  
}
