"""A DOM walker that fires SAX events, written as a SAX 2.0 XMLReader.

$Id: dom2sax.py,v 1.5 2001/08/19 19:27:40 larsga Exp $
"""

from xml.sax.saxutils import XMLGenerator
from xml.sax.xmlreader import *
from xml.sax.handler import *
from xml.sax._exceptions import *

# --- Useful constants

ELEMENT_NODE                = 1
ATTRIBUTE_NODE              = 2
TEXT_NODE                   = 3
CDATA_SECTION_NODE          = 4
ENTITY_REFERENCE_NODE       = 5
ENTITY_NODE                 = 6
PROCESSING_INSTRUCTION_NODE = 7
COMMENT_NODE                = 8
DOCUMENT_NODE               = 9
DOCUMENT_TYPE_NODE          = 10
DOCUMENT_FRAGMENT_NODE      = 11
NOTATION_NODE               = 12


# --- The walker

class DOMReader(XMLReader):

    def __init__(self, node = None):
        XMLReader.__init__(self)
        self._node = node
        self._processing = 0

        
    # --- XMLReader methods
        
    def getProperty(self, name):
        if name == property_dom_node:
            return self._node

        raise SAXNotRecognizedException("Property '%s' not recognized"
                                        % name)
        
    def setProperty(self, name, value):
        if name == property_dom_node:
            if self._processing:
                raise SAXNotSupportedException("The DOM node cannot "
                                       "be changed during processing")
            
            self._node = value
            return

        raise SAXNotRecognizedException("Property '%s' not recognized"
                                        % name)


    def parse(self, source):
        self._processing = 1
        try:
            self._cont_handler.startDocument()
            self._process_node(self._node)
        finally:
            self._cont_handler.endDocument()
            self._processing = 0


    # --- Internal methods

    def _process_node(self, node):
        old = self._node
        self._node = node
        type = node._get_nodeType()
        name = node._get_nodeName()
        
        if type == ELEMENT_NODE:
            self._cont_handler.startElement(name,
                                DOMAttributes(node._get_attributes()))
            for ch in  node._get_childNodes():
                self._process_node(ch)

            self._node = node
            self._cont_handler.endElement(name)

        elif type == TEXT_NODE or type == CDATA_SECTION_NODE:
            self._cont_handler.characters(node._get_nodeValue())

        elif type == PROCESSING_INSTRUCTION_NODE:
            self._cont_handler.processingInstruction(name,
                                               node._get_nodeValue())

        elif type == DOCUMENT_NODE or type == DOCUMENT_FRAGMENT_NODE:
            for ch in  node._get_childNodes():
                self._process_node(ch)

        elif type < 1 or type > 12:
            self._err_handler.warning(SAXException("Unknown node type: "
                                                   + str(node)))
        self._node = old


# --- DOMAttributes

class DOMAttributes:

    def __init__(self, map):
        self._map = map

    def getLength(self):
        return len(self._map)

    def getType(self, name):
        return "CDATA"

    def getValue(self, name):
        return self._map[name]._get_nodeValue()

    def getValueByQName(self, qname):
        return self._map[qname]._get_nodeValue()

    def getNameByQName(self, qname):
        raise NotImplementedError("Not yet implemented!")
    
    def getQNameByName(self, name):
        raise NotImplementedError("Not yet implemented!")
    
    def getNames(self):
        return self._map.keys()
    
    def getQNames(self):
        return self._map.keys()
    
    def __len__(self):
        return len(self._map)

    def __getitem__(self, name):
        return self._map[name]._get_nodeValue()

    def keys(self):
        return self._map.keys()

    def has_key(self, name):
        return self._map.has_key(name)

    def get(self, name, alternative=None):
        return self._map.get(name, alternative)
        
    def copy(self):
        return self.__class__(self._map)

    def items(self):
        items = []
        for (name, attr) in self._map.items():
            items.append((name, attr._get_nodeValue()))
        return items

    def values(self):
        values = []
        for attr in self._map.values():
            values.append(attr._get_nodeValue())
        return values


# --- Test

if __name__ == "__main__":
    import sys
    if sys.platform[ : 4] == "java":
        import javadom
        doc = \
         javadom.XercesDomImplementation().buildDocumentFile(sys.argv[1])
    else:
        from xml.dom.ext.reader.Sax2 import Reader
        doc = Reader().fromStream(open(sys.argv[1]))

    from xml.sax import saxutils
    walker = DOMReader(doc)
    walker.setContentHandler(XMLGenerator(open("out.xml", "w")))
    walker.parse(doc)

